<?php
namespace Modules\Cotacao\Modules\PlanilhaCotacao\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Modules\Cotacao\Modules\PlanilhaCotacao\Services\Sup05200PlanilhaDeCotacaoService;

class Sup05200PlanilhaDeCotacaoController extends \Zeedhi\Framework\Controller\Report
{

    protected $newCode;
    protected $environment;
    
    protected $errors = array();
    protected $errorMessages = array();
    
    public function __construct($newCode, $environment, Sup05200PlanilhaDeCotacaoService $service)
    {
        $this->newCode       = $newCode;
        $this->environment   = $environment;
        $this->service       = $service;
    }
    
    public function generateCode($newCode) {
        return $this->newCode->getCodeByTableName($newCode, $this->environment->getNrOrgTrab());
    }
    
    //LOG DE ERRO
    private function addError($errorType, $forn, $index = 0) {
        if (!isset($this->errors[$index])) {
            $this->errors[$index] = array();
        }
        if (!isset($this->errors[$index][$errorType])) {
            $this->errors[$index][$errorType] = array();
        }
        array_push($this->errors[$index][$errorType], $forn);
    }
    
    public function buscaDadosFiltro(DTO\Request $request, DTO\Response $response) {
        try {
            
            $central = $this->service->getcentralCompras();
            if(!$central)
                throw new \Exception("CDFILCENTCMP_NOT_REGISTERED");
                
            $params = $this->service->buscaParametros($central);
            if(empty($params))
                throw new \Exception("NAO_HA_PARAM_CADASTRADOS");
            
            
            $comprador = $this->service->verificaOperadorComprador();
            /*if(empty($comprador))
                throw new \Exception("OPERADOR_NAO_RELACIONADO");*/
            
            $comprador = !empty($comprador) ? $comprador[0] : $comprador;
            
            $datas = $this->service->buscaData($central);
            
            $datas = !empty($datas) ? $datas[0] : $datas;
            
            
            $response->addDataSet(new DataSet("resp", [$datas, $comprador, $central, $params]));    
            
        
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function validationBeforeMoveRow(DTO\Request $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            
            $operadores = $this->service->operadoresRelacionados($row);
        
            if($row['CDOPERADOR'] != $this->environment->getCdoperador()){
                if(!$this->operadorExistsInArray($operadores, $row)){
                    throw new \Exception("OPERADOR_NAO_RELACIONADO");
                }
            }
            
            if($row['IDUTINDICECOL'] == 'S') {
                if(count($this->service->verifyIndiceFinanceiro($row)) > 0){
                    throw new \Exception("INDICE_DEVE_SER_INFORMADO");               
                }
            }
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function verificaOperadorRelacionado(DTO\Request $request, DTO\Response $response)
    {
        $row = $request->getRow();
            
        $operadores = $this->service->operadoresRelacionados($row);
        
        if($row['CDOPERADOR'] != $this->environment->getCdoperador()){
            if(!$this->operadorExistsInArray($operadores, $row)){
                $response->setError(new Error("Operador não relacionado ao comprador.", 1));
            }
        }
    }
    
    public function validaDeletaCotacao(DTO\Request $request, DTO\Response $response){
        try {
            $row = $request->getRow();
            
            $operadores = $this->service->operadoresRelacionados($row);
            
            if(empty($operadores)){
                    
                $comprador = $this->service->verificaOperadorComprador();
    
                if(empty($comprador)){
                    throw new \Exception("Operador do sistema não é um comprador.");
                }else {
                    if($row['IDUMPRODCMP'] != 'S'){
                    
                        if($this->environment->getCdoperador() != $row['CDOPERADOR']){
                            throw new \Exception("Cotação pertence a outro comprador.");
                        }
                    }
                }
            }else{
                $operador = $this->operadorExistsInArray($operadores, $row);
                if($operador){
                    if($operador['IDCOTCMPOPER'] == 'N'){
                        //$response->setError(new Error("Operador não autorizado a manipular cotações deste comprador.", 1));
                        throw new \Exception("Operador não autorizado a manipular cotações deste comprador.");
                    }
                }else{
                    //$response->setError(new Error("Operador não relacionado ao comprador.", 1));
                    throw new \Exception("Operador não relacionado ao comprador.");
                }
            }
            
            $this->verificaCotacaoNaListaPreco($response, $row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    private function decodeErrorMessages() {
        $tab = "          ";
        foreach ($this->errors as $rowNum => $error) {
            if (isset($error["Valida_Insert_msg_01"])) {
                $message = "Fornecedor não foi incluído devido ao(s) seguinte(s) motivo(s):";
                array_push($this->errorMessages, $message);
            }
            if (isset($error["Valida_Insert_msg_02"])) {
                $message = "Alguns produtos não foram incluídos devido ao(s) seguinte(s) motivo(s):";
                array_push($this->errorMessages, $message);
            }
            if (isset($error["N_FornCorr_Cad"])) {
                foreach (($error["N_FornCorr_Cad"]) as $forn) {
                    $message = "Fornecedor " . $forn . " inativo ou não qualificado.";
                    array_push($this->errorMessages, $message);
                }
            }
            if (isset($error["MensErro_1"])) {
                foreach (($error["MensErro_1"]) as $forn) {
                    $message = "Fornecedor " . $forn . " não atende às regiões da cotação.";
                    array_push($this->errorMessages, $message);
                }
            }
            if (isset($error["MensErro_2"])) {
                foreach (($error["MensErro_2"]) as $forn) {
                    $message = "Fornecedor " . $forn . " desqualificado para o produto da cotação.";
                    array_push($this->errorMessages, $message);
                }
            }
            if (isset($error["MensErro_3"])) {
                foreach (($error["MensErro_3"]) as $forn) {
                    $message = "Fornecedor " . $forn . " não cadastrado para o produto da cotação.";
                    array_push($this->errorMessages, $message);
                }
            }
            if (isset($error["Forn_N_Cad_Prod"])) {
                foreach (($error["Forn_N_Cad_Prod"]) as $forn) {
                    $message = "Fornecedor informado não cadastrado ou não qualificado para o produto " .$forn. " da cotação.";
                    array_push($this->errorMessages, $message);
                }
            }
            if (isset($error["NProd_Cmp"])) {
                $message = "Não há produto de compra para o(s) produto(s) informado(s).";
                array_push($this->errorMessages, $message);
            }
            if (isset($error["N_Acomprar"])) {
                $message = "Não há solicitação de compras para os dados informados.";
                array_push($this->errorMessages, $message);
            }
            if (isset($error["N_FornProdCmp"])) {
                $message = "Produto(s) informado(s) não cadastrado(s) para o comprador e/ou fornecedor(es) informado(s).";
                array_push($this->errorMessages, $message);
            }
            if (isset($error["N_ProdComprador"])) {
                $message = "Produto(s) informado(s) não cadastrado(s) para o comprador informado";
                array_push($this->errorMessages, $message);
            }
            if (isset($error["N_MarcProd"])) {
                $message = "Marca(s) informada(s) não cadastrada(s) ou não qualificada(s) para o(s) produto(s) informado(s).";
                array_push($this->errorMessages, $message);
            }
            if (isset($error["N_QualifMarcProd"])) {
                $message = "Marca(s) informada(s) não qualificada(s) para o(s) produto(s) informado(s).";
                array_push($this->errorMessages, $message);
            }
            if (isset($error["N_QfoProdForn"])) {
                $message = "Produto(s) informado(s) não qualificado(s) para o(s) fornecedor(es) informado(s).";
                array_push($this->errorMessages, $message);
            }
            if (isset($error["N_RefoReg_Inc"])) {
                $message = "Fornecedor(es) não atende(m) á(s) região(ões) informada(s).";
                array_push($this->errorMessages, $message);
            }
            if (isset($error["Pr_NFilCor"])) {
                $message = "Produto(s) não negociado(s) pela unidade corporativa.";
                array_push($this->errorMessages, $message);
            }
            if (isset($error["Pr_NFilReg"])) {
                $message = "Produto(s) não negociado(s) pela unidade regional.";
                array_push($this->errorMessages, $message);
            }
            if (isset($error["Produto_Inativo"])) {
                $message = "Produto(s) informado(s) inativo(s).";
                array_push($this->errorMessages, $message);
            }
            if (isset($error["Valid_Venc"])) {
                $message = "Fornecedor com a validade vencida no período da Cotação.";
                array_push($this->errorMessages, $message);
            }
        }
    }
    
    public function salvaEntraListaGrid(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        
        foreach ($row as $cotacao) {
            $cotacao['DTCONCOLEPREC'] = null;
            $this->service->salvaCotacaoEditada($cotacao);
        }
    }
    
    public function operadorExistsInArray($operArray, $row){
        $occurrences = 0;
        
        for($i = 0; $i < count($operArray); $i++){
            
            if (in_array($this->environment->getCdoperador(), $operArray[$i])) { 
                return $operArray[$i];
            }else{
                $occurrences += 1;
            }
        }

        // if ($occurrences == count($operArray) {
        if ($occurrences == ((is_array($operArray) && !is_null($operArray)) ? count($operArray) : 0)) {
            return false;
        }
    }
    
    public function confirmaValidaGerente(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            
            $senha = $this->service->senhaGerente($row)[0];
            if($senha && md5($row["SENHA"]) != $senha['CDSENHAOPERWEB']){
                $response->setError(new Error("Senha inválida!", 1));
                return false;
            }else{
                return true;
            }
                
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function salvaCotacaoEditada(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            
            $situacaoColeta = $this->service->validaSituacaoColeta($row)[0];
            //if($situacaoColeta['IDSITUCOLETA'] == 'C' && $situacaoColeta['IDSITUCOLETA'] != $row['IDSITUCOLETA']){
                
                if(in_array('SENHA', array($row))) {
                    if($row['SENHA']){
                        if($this->confirmaValidaGerente($request, $response)){
                            $this->service->salvaCotacaoEditada($row);
                        }else{
                            $response->setError(new Error("Senha inválida!", 1));
                        }
                    }else{
                        $response->setError(new Error("Senha inválida!", 1));
                    }
                } else {
                    $this->service->salvaCotacaoEditada($row);
                }
            //}
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function deletaCotacao(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            
            $this->service->deletaCotacao($row);
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function verificaCotacaoNaListaPreco($response, $row){
        $countLista = $this->service->listaPreco($row)[0];
        if($countLista['CONTA'] > 0){
            $response->addDataSet(new DataSet("resp", ['S']));
        }else{
            $response->addDataSet(new DataSet("resp", []));
            $this->service->deletaCotacao($row);
        }
        
    }
    
    public function salvaCotacao(DTO\Request $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            $verificaComprador = $this->service->validaCadastroComprador($row);
            if(!empty($verificaComprador)){
                $NRCOLEPREC = $this->generateCode('COLETA'.$row['CDFILIAL']);
                $paraSupri = $this->service->getParaSupri($row);
                
                if(!empty($paraSupri) && $paraSupri[0]['IDUTISTATCOT'] == 'S'){
                    $row['IDSITUCOLETA'] = 'A';
                }else{
                    $row['IDSITUCOLETA'] = '';
                }
                if(!empty($paraSupri) && $paraSupri[0]['IDUTINDICECOL'] == 'N'){
                    $row['CDTPINDICE'] = null;
                    $row['IDPERVALOR'] = null;
                    $row['CDINDICE'] = null;
                    $row['DTREFIND'] = null;
                }
                $this->service->salvaCotacao($row, $NRCOLEPREC);
                
                if((!$row['NRCOMPRA'] || empty($row['NRCOMPRA']))){
                    if ($row['IDTIPCOTACAO'] == 'A') {
                        $rowsAffect = $this->service->procIncluiItemcolTodos($row, $NRCOLEPREC);
                    } else {
                        $rowsAffect = $this->service->procIncluiItemcolAComprar($row, $NRCOLEPREC);
                    }
                }else{
                    $rowsAffect = $this->service->procIncluiItemcolProdCompra($row, $NRCOLEPREC); 
                }
                
                if($row['IDCARGOBS'] == 'S'){
                    $this->service->procCargDescSolCot($row, $NRCOLEPREC);
                }
                
                if(!empty($rowsAffect) && $rowsAffect['P_ROWSAFFECT'] == 'N'){
                    self::validaInclusaoCotacao($row);
                    if (!empty($this->errors)) {
                        $this->decodeErrorMessages();
                        $response->addDataSet(new DataSet("errors", $this->errorMessages));
                        throw new \Exception("sheetErrors", 1);
                    }
                }
            }else{
                throw new \Exception("Comprador não cadastrado!", 1);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function validaInclusaoCotacao($row){
        self::validaCamposMarcFornReg($row);
        
        $paraSupri = $this->service->getParaSupri($row);
        if(!empty($paraSupri) && $paraSupri[0]['IDUTLPROCMP'] == 'S'){
            $prodCmp = $this->service->validaProdCmp($row);
            if(empty($prodCmp)){
                $this->addError("NProd_Cmp", '');
            }
        }
        
        if($row['IDTIPCOTACAO'] == 'C'){
            $solicita = $this->service->getSolicita($row);
            if(empty($solicita) || $solicita[0]['QTACOMPRAR'] == 0 ){
                $this->addError("N_Acomprar", '');
            }
        }
        
        $solicita = $this->service->getFornProdCmp($row);
        if(empty($solicita)){
            $this->addError("N_FornProdCmp", '');
        }
        
        $prodComp = $this->service->getProdComp($row);
        if(empty($prodComp)){
            $this->addError("N_ProdComprador", '');
        }
        
        if(!empty($paraSupri) && empty($paraSupri[0]['CDMARCA'])){
            $marcProd = $this->service->getMarcProd($row);
            if(empty($marcProd)){
                $this->addError("N_MarcProd", '');
            }
        }
        
        $qualiMarcProd = $this->service->validaQualificaMarcProd($row);
        if(empty($qualiMarcProd)){
            $this->addError("N_QualifMarcProd", '');
        }
        
        $qfoProdForn = $this->service->validaQfoProdForn($row);
        if(empty($qfoProdForn)){
            $this->addError("N_QfoProdForn", '');
        }
        
        $reFoReg = $this->service->getRegFoReg($row);
        if(empty($reFoReg)){
            $this->addError("N_RefoReg_Inc", '');
        }
        
        $funcFili = $this->service->validaFuncFili($row);
        if(!empty($funcFili) && $funcFili[0]['IDFUNCAOFILI'] == 'C'){
            $filiCor = $this->service->validaFiliCor($row);
            if(empty($filiCor)){
                $this->addError("Pr_NFilCor", '');
            }
        }
        else if(!empty($funcFili) && $funcFili[0]['IDFUNCAOFILI'] == 'R'){
            $filiReg = $this->service->validaFiliReg($row);
            if(empty($filiReg)){
                $this->addError("Pr_NFilReg", '');
            }
        }
        
        $prodInativo = $this->service->getProdInativo($row);
        if(empty($prodInativo)){
            $this->addError("Produto_Inativo", '');
        }
        
        $dtQualifForn = $this->service->getDtQualifForn($row);
        if(empty($dtQualifForn)){
            $this->addError("Valid_Venc", '');
        }
    }
    
    public function validaCamposMarcFornReg($row){
        $row['CDFORNECED'] = $row['CDFORNECED'] == null ? 'T' : $row['CDFORNECED'];
        $row['CDMARCA']    = $row['CDMARCA']    == null ? 'T' : $row['CDMARCA'];
        $row['CDREGIAO']   = $row['CDREGIAO']   == null ? 'T' : $row['CDREGIAO'];
    }
    
    public function confirmaValidaComprador(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            
            $row['CDGERENTE'] = $row['CDCOMPRADOR'];
            $senha = $this->service->senhaGerente($row)[0];
            if($senha && md5($row["SENHA"]) != $senha['CDSENHAOPERWEB']){
                $response->setError(new Error("Senha inválida!", 1));
            }  
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function excluiItemPrecoZerado(DTO\Request $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            $precoZerado = $this->service->verificaPrecoZerado($row)[0];
            if($precoZerado['CTTOTITM'] == $precoZerado['CTTOTZERO'] ){
                throw new \Exception("Operação não permitida! Não existe nenhum preço informado.", 1);
            }
                
            $this->service->deleteItemPrecoZerado($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function carregarQtdeComprar(DTO\Request $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            $this->service->carregarAComprar($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function atualizarFormaPagamento(DTO\Request $request, DTO\Response $response){
        try{
            $row = $request->getRow();

            $this->service->updateItem($row);
            $this->service->deletaPrazoItemCol($row);
            
            $itensColeta = $this->service->verificaItemCole($row);
            
            foreach ($itensColeta as $item){
                $this->preparaInsertFormPgto($item, $item, $item);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function salvaRegiao(DTO\Request $request, DTO\Response $response){
        try{
            $row = $request->getRow();

            $regiaoExists = $this->service->verificaRegiaoExiste($row);
            if(empty($regiaoExists)){
                $this->validaRegiaoFornecedor($row);
                
                if (!empty($this->errors)) {
                    $this->decodeErrorMessages();
                    $response->addDataSet(new DataSet("errors", $this->errorMessages));
                    throw new \Exception("sheetErrors", 1);
                }
            }else{
                throw new \Exception("Região já cadastrada", 1);
            }
                
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function validaRegiaoFornecedor($row){
        //Verifica se o fornecedor atende a região
        $fornecedores = [];
        if($row['CDFORNECED']){
            $fornecedores[0] = [
                'CDFORNECED' => $row['CDFORNECED'],
                'NMFANTFORN' => $row['NMFANTFORN']
            ];
        }else{
            $fornecedores = $this->service->getFornecedoresCotacao($row);
        }
        $fornAtend = array();
        $cont = 0;
        for($i = 0; $i < count($fornecedores); $i++){
            $situacaoForn = $this->service->verificaSituacaoForn($fornecedores[$i], $row);
            $fornReg      = $this->service->verificaRegForn($fornecedores[$i], $row);
            if(!empty($situacaoForn)){
                $cont += 1;
            }else{
                
            }
            if(empty($fornReg)){
                array_push($fornAtend, $fornecedores[$i]);
            }else{
                $this->addError("MensErro_1", $fornecedores[$i]['NMFANTFORN']);
            }
        }
        // if(count($fornAtend) == count($fornecedores)){
        //     throw new \Exception("Região não é atendida por nenhum fornecedor da cotação");
        // }
        
        if($cont == 0 && count($fornecedores) > 1){
            throw new \Exception("Não há fornecedor ativo e qualificado para a cotação.");
        }
        
        if(!empty($fornAtend)){
            $produtos = $this->service->buscaProdutoCotacao($row);
            
            foreach ($fornAtend as $vdForn) {
                $prodM = $this->service->getProdM($row, $vdForn['CDFORNECED']);
                if(empty($prodM)){
                    $this->addError("MensErro_3", $vdForn['NMFANTFORN']);
                }else{
                    foreach ($prodM as $prod) {
                        $prodForn = $this->service->verifyProdForn($row, $prod, $vdForn)[0];
                        if(empty($prodForn)){
                            $this->addError("MensErro_2", $vdForn['NMFANTFORN']);
                        }else{
                            $precUnit = 0;
                            if(!empty($row['CDREGIAOPREC'])){
                                $precUnit  = $this->service->getPrecoUnitario($row, $prod, $vdForn);
                            }
                            $listaPrec = $this->service->getListaPreco($row)[0];
                            $arrayToInsert = $this->preparaArray($row, $vdForn, $prod, $listaPrec, $precUnit);
                            
                            $this->service->insertItemColRegiao($arrayToInsert, $row);
                            
                            $this->preparaProcImposto($row, $vdForn, $prod);
                            $this->preparaInsertFormPgto($row, $vdForn, $prod);
                        }
                    }
                }
            }
        }
    }
    
    public function preparaArray($row, $vdForn, $prodM, $listaPrec, $precUnit){
        $countPrecUnit = 0;
        if (is_array($precUnit)){
            $countPrecUnit = count($precUnit);
        } 
        
        $arr = [
            "NRSEQPRELIST"    => $listaPrec['NRSEQPRELIST'],
            "IDITVALDIFLISTA" => $listaPrec['IDITVALDIFLISTA'] ? $listaPrec['IDITVALDIFLISTA'] : 'N',
            "IDPRECALT"       => $listaPrec['IDPRECALT']       ? $listaPrec['IDPRECALT']       : 'N',
            "CDFILIAL"        => $row['CDFILIAL'],
            "NRCOLEPRECO"     => $row['NRCOLEPRECO'],
            "CDPRODUTO"       => $prodM['CDPRODUTO'],
            "CDMARCA"         => $prodM['CDMARCA'],
            "CDFORNECED"      => $vdForn['CDFORNECED'],
            "CDREGIAO"        => $row['CDREGIAO'],
            "VRPRECUNIITM"    => $countPrecUnit == 0 ? $countPrecUnit : $precUnit[0]['VRPRECUNIITM'],
            "VRINDICEITMCOL"  => $countPrecUnit == 0 ? $countPrecUnit : $precUnit[0]['VRINDICEITMCOL'],    
            "CDOPERINCL"      => $row['CDOPERINCL'],
            "IDATUREG"        => null,
            "IDATUFORN"       => null,
        ];
        return $arr;
    }
    
    public function preparaProcImposto($row, $vdForn, $prodM){
        $paramEmitenteAf = [
            "CDFILIAL"  => $row['CDFILIAL'],
            "CDPRODUTO" => $prodM['CDPRODUTO']
        ];
        
        $codEmpresa = $this->service->getEmitenteAF($paramEmitenteAf);
        if(empty($codEmpresa) || $codEmpresa[0]['CDEMPRESA'] == null){
            $codEmpresa = $this->service->getEmitentePadrao($paramEmitenteAf);
        }
        if(empty($codEmpresa) || $codEmpresa[0]['CDEMPRESA'] == null){
            $codEmpresa = $this->service->getEmpresaFili($paramEmitenteAf);
        }
        
        $infoEmpresa = $this->service->getEmpresaInfo($codEmpresa[0])[0];
        
        $arrayToProc = $this->preparaArrayProcImposto($row, $vdForn, $prodM, $infoEmpresa);
        
        $this->service->procImpostoItemcol($arrayToProc);
    }
    
    public function preparaArrayProcImposto($row, $vdForn, $prodM, $infoEmpresa){
        $arr = [
            "CDEMPRESA"   => $infoEmpresa['CDEMPRESA'],
            "CDINSCESTA"  => $infoEmpresa['CDINSCESTA'],
            "CDFORNECED"  => $vdForn['CDFORNECED'],
            "CDFILCENT"   => $row['CDFILIAL'],
            "NRCOLEPRECO" => $row['NRCOLEPRECO'],
            "CDFORNECED"  => $vdForn['CDFORNECED'],
            "CDREGIAO"    => $row['CDREGIAO'],
            "CDPRODUTO"   => $prodM['CDPRODUTO'],
            "CDMARCA"     => $prodM['CDMARCA'],
            "DTCOLEPREC"  => $row['DTCOLEPREC']
        ];
        
        return $arr;
    }
    
    public function preparaInsertFormPgto($row, $vdForn, $prodM){
        $params = [
            "CDFILIAL"    => $row['CDFILIAL'],
            "NRCOLEPRECO" => $row['NRCOLEPRECO'],    
            "CDPRODUTO"   => $prodM['CDPRODUTO'],    
            "CDMARCA"     => $prodM['CDMARCA'],    
            "CDFORNECED"  => $vdForn['CDFORNECED'],    
            "CDREGIAO"    => $row['CDREGIAO']    
        ];
        $contPzoCot = $this->service->getContPzoCot($params)[0];
        if($contPzoCot['CTPZOS'] == 0){
            $pgtoProdForn = $this->service->getPgtoProdForn($params);
            if(empty($pgtoProdForn) || $pgtoProdForn[0]['IDTPFORPGPRF'] == ''){
                
                $pgtoForn = $this->service->getPgtoForn($vdForn)[0];
                
                $paramUpPzPgtCol = [
                    "CDFORMPGTO"   => $pgtoForn['CDFORMPGTO'],
                    "IDTPFOPGTCOL" => $pgtoForn['IDTPFORPGTFO'],
                    "NRDIAPGFXCOL" => $pgtoForn['NRDIAPGFIXFO'],
                    "NRPARCPGTCOL" => $pgtoForn['NRPARCPGTOFO'],
                    "IDUTDIAPGCOL" => $pgtoForn['IDUTDIAPGTFO']
                ];
                $this->service->upPrazoPagamentotCol($params, $paramUpPzPgtCol);
                
                if($pgtoForn['IDTPFORPGTFO'] == 'V' ){
                    $this->service->deleteFornecedorRegister($params);
                    
                    $parcelas = $this->service->getParcelas($params);
                    if (!empty($parcelas)) {
                        foreach ($parcelas as $parcela) {
                            $this->service->insertFornecedor($parcela);
                        }
                    }
                    
                    $this->service->forcTrigger($params);
                }
            }else{
                $paramUpPzPgtCol = [
                    "CDFORMPGTO"   => $pgtoProdForn[0]['CDFORMPGTO'],
                    "IDTPFOPGTCOL" => $pgtoProdForn[0]['IDTPFORPGPRF'],
                    "NRDIAPGFXCOL" => $pgtoProdForn[0]['NRDIAPGFIPRF'],
                    "NRPARCPGTCOL" => $pgtoProdForn[0]['NRPARCPGTPRF'],
                    "IDUTDIAPGCOL" => $pgtoProdForn[0]['IDUTDIAPGPRF'],
                    "PRODFORN"     => $pgtoProdForn[0]['CDPRODUTO']
                ];
                $this->service->upPrazoPagamentotCol($params, $paramUpPzPgtCol);
                if($pgtoProdForn[0]['IDTPFORPGPRF'] == 'V'){
                    $this->service->insertProdutoFornecedor($params, $paramUpPzPgtCol);        
                }
                $this->service->forcTrigger($params);
            }
        }
    }
    
    public function salvarFornecedor(DTO\Request $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            $this->validaFornecedor($row);
            $procResult = $this->service->procIncFornCota($row);
            if($procResult['P_IDERRO'] != "000"){
                $logErro = $this->service->logErros($row);
                $response->addDataSet(new DataSet("errors", $logErro));
                $this->service->deleteLog($row);
                throw new \Exception("sheetErrors", 1);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function validaFornecedor($row){
        $validaFornCot = $this->service->validaFornCot($row)[0];
        if (!in_array($row['CDFORNECED'], $validaFornCot)) {
            throw new \Exception("Fornecedor " .$row['NMFANTFORN']. " não está cadastrado para a cotação.");
        }
    }
    
    public function salvarProduto(DTO\Request $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            
            $this->verifyBeforeInsert($row);
            
            $this->service->insertItemCole($this->preparaArrayProdutoInsert($row), $row);
            
            $this->preparaProcImposto($row, $row, $row);
            $this->preparaInsertFormPgto($row, $row, $row);
            $this->preparaProcPrecos($row);
            
            $this->service->updateParametrosCotacao($row);
            
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function preparaProcPrecos($row){
        if($row['IDATUREG']){
            $row['P_IDATUREG'] = 'T';
            
            if($row['IDATUFORN']){
                $row['P_IDATUFORN'] = 'T';
            }else{
                $row['P_IDATUFORN'] = 'S';
            }
            
            $row['NEW_CDREGIAO'] = null;
            $this->service->procPrecosToRegiao($row);
        }
        
        if($row['IDATUFORN']){
            $row['P_IDATUFORN'] = 'T';
            if($row['IDATUREG']){
                $row['P_IDATUREG'] = 'T';
            }else{
                $row['P_IDATUREG'] = 'S';
            }
            $this->service->procPrecosToForn($row);
        }
    }
    
    public function preparaArrayProdutoInsert($row){
        $listaPreco = $this->service->getListaPreco($row)[0];
        $arr = [
            "CDFILIAL"        => $row['CDFILIAL'], 
            "NRCOLEPRECO"     => $row['NRCOLEPRECO'], 
            "CDPRODUTO"       => $row['CDPRODUTO'],
            "CDMARCA"         => $row['CDMARCA'], 
            "CDFORNECED"      => $row['CDFORNECED'], 
            "CDREGIAO"        => $row['CDREGIAO'],
            "VRPRECUNIITM"    => !empty($row['VRPRECUNIITM']) ? (float)$row['VRPRECUNIITM'] : 0,
            "CDOPERINCL"      => $this->environment->getCdoperador(),
            "IDATUREG"        => $row['IDATUREG'],
            "IDATUFORN"       => $row['IDATUFORN'],
            "NRSEQPRELIST"    => $listaPreco['NRSEQPRELIST'],
            "IDITVALDIFLISTA" => $listaPreco['IDITVALDIFLISTA'] ? $listaPreco['IDITVALDIFLISTA'] : 'N',
            "IDPRECALT"       => $listaPreco['IDPRECALT'] ? $listaPreco['IDPRECALT'] : 'N'
        ];
        
        return $arr;
    }
    
    public function verifyBeforeInsert($row){
        $qualifMarca = $this->service->getQualifMarca($row);
        if(!empty($qualifMarca)){
            throw new \Exception("Marca não qualificada para o produto informado.");
        }
        
        $verQfoProdForn = $this->service->getVerQfoProdForn($row);
        if(empty($verQfoProdForn)){
            throw new \Exception("Produto não qualificado para o fornecedor.");
        }
        
        $funcFili = $this->service->validaFuncFili($row);
        if($funcFili && $funcFili[0]['IDFUNCAOFILI'] == 'R'){
            $filiReg = $this->service->getFiliReg($row);
            if(empty($filiReg)){
                throw new \Exception("Produto não negociado pela regional.");
            }
        }else if($funcFili && $funcFili[0]['IDFUNCAOFILI'] == 'C'){
            $filiCor = $this->service->getFiliCor($row);
            if(empty($filiCor)){
                throw new \Exception("Produto não negociado pela regional.");
            }
        }
        
        $prodFornComp = $this->service->validaProdutoFornecedor($row);
        if(empty($prodFornComp)){
            throw new \Exception("Produto não cadastrado para o fornecedor.");
            
        }
        
        $marcProd = $this->service->validaMarcaProduto($row);
        if(empty($marcProd)){
            throw new \Exception("Marca não cadastrada para o produto.");
        }
        
        $obrigaEAN = $this->service->getCodEAN($row);
        if($obrigaEAN && $obrigaEAN[0]['IDOBRIGAEAN'] == 'S'){
            $existEAN = $this->service->verificaExisteEAN($row);
            if(empty($existEAN)){
                throw new \Exception("Código EAN não cadastrado para o produto/marca do item de cotação");
            }
        }
    }
    
    public function verificaLista(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        
        $verifLista = $this->service->verificaLista($row)[0];
        
        if($verifLista['CONTA'] > 0){
            $response->addDataSet(new DataSet("resp", [true]));
        }else{
            if($row['REGI'] == 'R'){
                $this->deletaRegiao($request, $response);
            }else if($row['REGI'] == 'F'){
                $this->deletaFornecedor($request, $response);
            }else{
                $this->deletaProduto($request, $response);
            }
        }
        if (!empty($this->errors)) {
            $this->decodeErrorMessages();
            $response->addDataSet(new DataSet("errors", $this->errorMessages));
            throw new \Exception("sheetErrors", 1);
        }
    }
    
    public function deletaRegiao(DTO\Request $request, DTO\Response $response){
        try{    
            $row = $request->getRow();
            
            $prodInfo = $this->prodInfoToDelete($row);
            
            $this->service->deletaRegiao($row);
            foreach ($prodInfo as $prod) {
                $this->service->procItemcoleAtuCustosup($prod);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function deletaFornecedor(DTO\Request $request, DTO\Response $response){
        try{    
            $row = $request->getRow();
            
            $prodInfo = $this->prodInfoToDelete($row);

            $row['CDREGIAO'] = $row['CDREGIAO'] != null ? $row['CDREGIAO'] : 'T'; 
            
            $this->service->deletaFornecedor($row);
            foreach ($prodInfo as $prod) {
                $this->service->procItemcoleAtuCustosup($prod);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function deletaProduto(DTO\Request $request, DTO\Response $response){
        try{    
            $row = $request->getRow();
            foreach ($row as $produto) {
                $prodInfo = $this->prodInfoToDelete($produto);
                $produto['CDREGIAO'] = $produto['CDREGIAO'] != null ? $produto['CDREGIAO'] : 'T'; 
                $this->service->deletaProduto($produto);
                foreach ($prodInfo as $prod) {
                    $this->service->procItemcoleAtuCustosup($prod);
                }
            }
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function prodInfoToDelete($row){
        $params = [
            "CDFILIAL"    => $row['CDFILIAL'],
            "NRCOLEPRECO" => $row['NRCOLEPRECO'],
            "CDPRODUTO"   => $row['CDPRODUTO']  != null ? $row['CDPRODUTO']  : 'T',
            "CDREGIAO"    => $row['CDREGIAO']   != null ? $row['CDREGIAO']   : 'T',
            "CDFORNECED"  => $row['CDFORNECED'] != null ? $row['CDFORNECED'] : 'T',
            "CDMARCA"     => $row['CDMARCA']    != null ? $row['CDMARCA']    : 'T'
        ];

        $prodInfo = $this->service->getProdInfoToDelete($params);
        
        return $prodInfo;
    } 
    
    public function carregarMelhoresPrecos(DTO\Request $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            
            $this->service->procMelhorPrecoAuto($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function reajustarPrecos(DTO\Request $request, DTO\Response $response)
    {
        try {
            $this->service->beginTransaction();
            $row = $request->getRow();
            $row['DSRDMUSUMCOT'] = null;
            $row['DSENDUSUMCOT'] = null;
            $row['DSNMUSUMCOT'] = null;
            $row['DSMETUSUMCOT'] = null;
            
            if ($row['REAJUSTE'] != 0) {
                $this->service->insertItemCotacaoReajuste($row);
                $this->service->updateDataPreco($row);

                $productsSupplyTek = $this->service->verifyProdOnSupplyTek($row, "multiple");
                if (sizeof($productsSupplyTek) > 0) {
                    $this->saveNotificationOnSupplyTek($row, $productsSupplyTek, "multiple");
                }
                
                $impostos = $this->service->getImpostoCotacao($row);
               
                foreach ($impostos as $imposto) {
                    $emitenteAf = $this->service->getEmitenteAF($imposto);
                    
                    if (empty($emitenteAf)) {
                        $emitenteAf = $this->service->getEmitentePadrao($imposto);
                    }
                    
                    $infoEmpresa = $this->service->getEmpresaInfo($emitenteAf[0]);
                        
                    $paramsProc = [
                        'CDEMPRESA'   => $infoEmpresa ? $infoEmpresa[0]['CDEMPRESA']  : null,
                        'CDINSCESTA'  => $infoEmpresa ? $infoEmpresa[0]['CDINSCESTA'] : null,
                        'CDFILCENT'   => $row['CDFILCENT'],
                        'NRCOLEPRECO' => $row['NRCOLEPRECO'],
                        'CDFORNECED'  => $row['CDFORNECED'],
                        'CDREGIAO'    => $row['CDREGIAO'],
                        'CDPRODUTO'   => $imposto['CDPRODUTO'],
                        'CDMARCA'     => $imposto['CDMARCA'],
                        'DTCOLEPREC'  => $row['DTCOLEPREC'],
                    ];
                    
                    $this->service->procImpostoItemcol($paramsProc);
                }
                
                $this->service->commitTransaction();
            }
        } catch (\Exception $e) {
            $this->service->rollbackTransaction();
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function salvarPrecoUnitario(DTO\Request $request, DTO\Response $response)
    {
        try {    
            $this->service->beginTransaction();
            $row = $request->getRow();
            
            foreach ($row as $prod) {
                $this->verifyBeforeInsert($prod);
                
                $prod['P_IDATUREG']  = $prod['IDATUREG']  ? 'T' : 'S';
                $prod['P_IDATUFORN'] = $prod['IDATUFORN'] ? 'T' : 'S';
                
                if ($prod['IDUTINDICECOL'] == 'S') {
                    $prod['VRINDICEITMCOL'] = (float)$prod['VRINDICEITMCOL'];
                } else {
                    $prod['VRPRECUNIITM'] = (float)$prod['VRPRECUNIITM'];
                }
                
                //$this->service->salvaCotacaoEditada($prod);
                
                if ($prod['DSJUSTITM']) {
                    $this->service->insertJustificativa($prod);
                }
                
                if(!empty($this->service->verificaFornAtende($prod))) {
                    throw new \Exception("Fornecedor não atende a região");
                } else {
                    if($prod['IDUTINDICECOL'] == 'S'){
                       $this->service->updRegFornIndice($prod);
                    } else {
                        $this->service->updateRegForn($prod);    
                    }
                
                    $this->preparaProcImposto($prod, $prod, $prod);
                    $this->preparaInsertFormPgto($prod, $prod, $prod);
                    
                    $this->preparaProcPrecos($prod);
                    
                    $this->service->updatePreEcZerado($prod);    
                }
                
                $productSupplyTek = $this->service->verifyProdOnSupplyTek($prod, "single");
                
                if (sizeof($productSupplyTek) > 0) {
                    $this->saveNotificationOnSupplyTek($prod, $productSupplyTek, "single");
                }
            }
            
            $this->service->updateParametrosCotacao($row[0]);
            $this->service->commitTransaction();
        } catch (\Exception $e) {
            $this->service->rollbackTransaction();
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    private function saveNotificationOnSupplyTek($row, $products, $type)
    {
        $row["NRNOTIFICSUPPLY"]   = $this->generateCode("NOTIFICSUPPLY");
        $row["CDFILIALSUPPLYTEK"] = $products[0]["CDFILIAL"];
        $row["CDOPERADOR"]        = $products[0]["CDOPERADOR"];
        
        if ($type == "single") {
            $row["NMNOTIFICACAO"] = "Alteração de Preço - " . $row["NMPRODUTO"];
            
            $justify = "";
            if ($row["DSJUSTITM_inline"] != "") {
                $justify = "com a seguinte justificativa: <b>\"" . $row["DSJUSTITM_inline"] ."\"</b> ";
            }
            
            $row["DSTEXTNOTIFIC"] = "<div>
                                    	<span> 
                                    		O produto " . $row["NMPRODUTO"] . " - " . $row["NMMARCA"] . " 
                                    		sofreu o seguinte reajuste: <b>" . $row["DSALTERITM"] . "</b> 
                                    		pelo operador <b>" . $this->environment->getCurrentUserName() . "</b> 
                                    		".$justify."
                                            na data: <b>" . date('d/m/Y') . "</b>
                                    	</span>
                                    </div>";
        } else {
            $row["NMNOTIFICACAO"] = "Alteração de Preços";
            
            $row["DSTEXTNOTIFIC"] = "<div>
                                    	<span> 
                                    		Os seguintes produtos tiveram seus preços reajustados em <b>" . $row["REAJUSTE"] . "</b>% 
                                    		pelo operador <b>" . $this->environment->getCurrentUserName() . "</b> 
                                    		na data: <b>" . date('d/m/Y') . "</b> 
                                    	</span>
                                        <ul>";
            
            foreach($products as $product) {
                $row["DSTEXTNOTIFIC"] .= "<li> " . $product["NMPRODUTO"] . " - " . $product["NMMARCA"] . " </li>";
            }
            
            $row["DSTEXTNOTIFIC"] .= "  </ul>
                                    </div>";
        }
        
        $this->service->insertNotificationSupplyTek($row);
    }
    
    public function carregaObservacao(DTO\Request $request, DTO\Response $response){
        try{    
            $row = $request->getRow();
            
            $observacao = $this->service->getObservacao($row);
            
            $response->addDataSet(new DataSet("resp", $observacao));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function salvarObservacao(DTO\Request $request, DTO\Response $response){
        try{    
            $row = $request->getRow();
            
            $this->service->salvarObservacao($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function validaOperadorComprador(DTO\Request $request, DTO\Response $response) 
    {
        try{    
            $row = $request->getRow();
            
            $response->addDataSet(new DataSet("response", $this->service->validaOperadorComprador($row)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    //TELA DUPLICAR COTAÇÃO
    
    public function duplicaCotacao(DTO\Request $request, DTO\Response $response){
        try{    
            $row = $request->getRow();
            if($row['CONSCOTAN'] == 'S'){
                $this->service->getConsCotAn($row);
            }
            
            $row['REAJUSTE'] = (float)$row['REAJUSTE'];
            $row['NRSEMAINIC'] = (string)$row['NRSEMAINIC'];
            $row['NRSEMAFINAL'] = (string)$row['NRSEMAFINAL'];
            
            $this->service->procDuplicaCotacao($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    //TELA ALTERAR VALIDADE COTAÇÃO
    
    public function alteraValidadeCotacao(DTO\Request $request, DTO\Response $response){
        try{    
            $row = $request->getRow();
            $cotacoesValidade = $this->service->getCotacoesValidade($row);
            foreach ($cotacoesValidade as $cotacao) {
                $row['CDFILIAL']    = $cotacao['CDFILIAL'];
                $row['NRCOLEPRECO'] = $cotacao['NRCOLEPRECO'];
                $this->service->updateValidadeColeta($row);
                $this->service->updateValidadeItemColeta($row);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    //TELA INFORMAÇÕES DA PLANILHA
    
    public function getFieldsValue(DTO\Request $request, DTO\Response $response){
        try{    
            $row = $request->getRow();
             
            $values = $this->service->getItemCole($row);
            $response->addDataSet(new DataSet("resp", $values));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        } 
    }
    
    public function salvarInfoPlanilha(DTO\Request $request, DTO\Response $response){
        try{    
            $row = $request->getRow();
            
            $this->validaInfoPlanilha($row);
            
            $this->preparaProcImposto($row, $row, $row);
            
            $this->service->deletaPrazoPgt($row);
            $this->service->updateItemCole($row);
            $this->service->updateFreteForn($row);
            
            $query = $this->generateUpdateQuery($row);
        
            $this->service->updateItemColePersonalizado($query, $row);
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        } 
    }
    
    public function validaInfoPlanilha($row){
        $coletaApr = $this->service->verificaColetaApr($row);
        if(empty($coletaApr)){
            throw new \Exception("Ocorreu um erro durante a confirmação das informações na planilha.");
        }
        
        $marca = $this->service->pesqMarca($row);
        if(empty($marca)){
            throw new \Exception("Marca não cadastrada para o produto.");
        }
        
        /*$marcaAutorizada = $this->service->getMarcaAutorizada($row);
        if(empty($marcaAutorizada)){
            throw new \Exception("Marca reprovada ou com validade vencida.");
        }*/
        
        if($row['CDTIPCOB']){
            $tipoCobranca = $this->service->getTipoCobranca($row);
            if(empty($tipoCobranca)){
                throw new \Exception("Tipo de cobrança não cadastrada.");
            }
        }
        
        $somaPercent = $this->service->getSomaPercent($row);
        if(!empty($somaPercent) && $somaPercent[0]['SOMAPERC'] != 100 && $somaPercent[0]['SOMAPERC'] != null){
            throw new \Exception("Soma do(s) percentual(is) do(s) prazo(s) de pagamento deve totalizar 100%.");
        }
    }
    
    public function generateUpdateQuery($row){
        $query = "UPDATE ITEMCOLE SET VRPERDESITM = :VRPERDESITM WHERE CDFILIAL = :OLD_CDFILIAL AND NRCOLEPRECO = :OLD_NRCOLEPRECO";
        if($row['REPLICADESCONTO'] == 'N'){
            if($row['IDDESCPROD'] == 1){
            $query = $query." AND CDPRODUTO = :P_CDPRODUTO";
            }else {
                $query = $query." AND INSTR(:P_CDPRODUTO, CDPRODUTO) > 0";
            }
            
            if($row['IDDESCMARCA'] == 1){
                $query = $query." AND CDMARCA = :P_CDMARCA";
            }else {
                $query = $query." AND INSTR(:P_CDMARCA, CDMARCA) > 0";
            }
            
            if($row['IDDESCREG'] == 1){
                $query = $query." AND CDREGIAO = :P_CDREGIAO";
            }else {
                $query = $query." AND INSTR(:P_CDREGIAO, CDREGIAO) > 0";
            }
            
            if($row['IDDESCFORN'] == 1){
                $query = $query." AND CDFORNECED = :P_CDFORNECED";
            }else {
                $query = $query." AND INSTR(:P_CDFORNECED, CDFORNECED) > 0";
            }
        }
        else
        {
            if($row['RDPROD'] == 'M'){
                $query = $query." AND CDPRODUTO = :P_CDPRODUTO";
            }else if ($row['RDPROD'] == 'T') {
                $query = $query." AND :P_CDPRODUTO = 'T'";
            }
            else{
                $query = $query." AND INSTR(:P_CDPRODUTO, CDPRODUTO) > 0";
            }
            
            if($row['RDMARCA'] == 'M'){
                $query = $query." AND CDMARCA = :P_CDMARCA";
            }else if ($row['RDMARCA'] == 'T') {
                $query = $query." AND :P_CDMARCA = 'T'";
            }
            else {
                $query = $query." AND INSTR(:P_CDMARCA, CDMARCA) > 0";
            }
            
            if($row['RDREGIAO'] == 'M'){
                $query = $query." AND CDREGIAO = :P_CDREGIAO";
            }else if ($row['RDREGIAO'] == 'T') {
                $query = $query." AND :P_CDREGIAO = 'T'";
            }
            else {
                $query = $query." AND INSTR(:P_CDREGIAO, CDREGIAO) > 0";
            }
            
            if($row['RDFORNECED'] == 'M'){
                $query = $query." AND CDFORNECED = :P_CDFORNECED";
            }else if ($row['RDFORNECED'] == 'T') {
                $query = $query." AND :P_CDFORNECED = 'T'";
            }
            else {
                $query = $query." AND INSTR(:P_CDFORNECED, CDFORNECED) > 0";
            }
        }
        return $query;
    }
    
    //FORMA DE PAGAMENTO
    
    public function saveFormaPagamento(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $this->service->updateItemCole($row[1]);
        if($row[1]['IDTPFORPGTFO'] == 'V'){
           
            $this->service->deletaPrazoPgt($row[1]);
            foreach ($row[0] as $data){
                $row[1]['NRDIASPGTOFO'] = $data['NRDIASPGTOFO'];
                $row[1]['VRPERCPGTOFO'] = $data['VRPERCPGTOFO'];
                $this->service->insertParcelas($row[1]);
            }
        }else{
           
            $this->service->deletaPrazoPgt($row[1]);
        }
    }
    
    //NEGOCIAÇÃO / SIMULAÇÃO
    
    public function buscaDadosHistoricoNegociacao(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        
        $itemCole = $this->service->getItemColeNegociacao($row)[0];

        $ultCotacaoForn        = $this->service->getUltimaCotacaoFornecedor($row);
        $ultCompra             = $this->service->getUltimaCompra($row);
        
        $row['DTINIVALITM'] = $itemCole['DTINIVALITM'];
        $row['DTFINVALITM'] = $itemCole['DTFINVALITM'];
        $row['IDTIPOFORN']  = 'N';
        $minPrecEcoNormal      = $this->service->getMenorPrecoEco($row);
        
        $row['IDTIPOFORN']  = 'A';
        $minPrecEcoAlternativo = $this->service->getMenorPrecoEco($row);
        
        $precOriginal    = [
            "VRPRECUNORIG" => $itemCole['VRPRECUNORIG'],
            "VRPRECECORIG" => $itemCole['VRPRECECORIG'],
            "VRPRECPIORIG" => $itemCole['VRPRECPIORIG']
        ];
        
        $prazoPgtoCol = $this->service->getPrazoPgtoItmCol($row);
        
        if(!empty($ultCompra)){
            $ultCompra = $ultCompra[0];
            $row['CDFILDEST']   = $ultCompra['CDFILDEST'];
            $row['CDFILLANC']   = $ultCompra['CDFILLANC'];
            $row['NRAUTOFORN']  = $ultCompra['NRAUTOFORN'];
            $row['DTPROGENTAF'] = $ultCompra['DTPROGENTAF'];
        }
        
        $procResult = $this->service->procPrecoEconAf($row);
        
        $dadosHistorico = [
            "ULTCOTFORN" => !empty($ultCotacaoForn)         ? $ultCotacaoForn[0]        : $ultCotacaoForn,
            "ULTCMP"     => $ultCompra,
            "MINPRECNOR" => !empty($minPrecEcoNormal)       ? $minPrecEcoNormal[0]      : $minPrecEcoNormal,
            "MINPRECALT" => !empty($minPrecEcoAlternativo)  ? $minPrecEcoAlternativo[0] : $minPrecEcoAlternativo,
            "PRAZPGTCOL" => !empty($prazoPgtoCol)           ? $prazoPgtoCol[0]          : $prazoPgtoCol,
            "ITEMCOLE"   => $itemCole,
            "PRECORI"    => $precOriginal,
            "PROC"       => $procResult
        ];
        
        $response->addDataSet(new DataSet("resp", $dadosHistorico));
    }
    
    public function calcularNegociacao(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        
        $query = $this->generateUpdateQuery($row);
        
        $this->service->updateItemColePersonalizado($query, $row);
        
        $this->preparaProcImposto($row, $row, $row);
        
        $row['NRSEQAPUR'] = ' ';
        $this->service->procAtualizaItmapur($row);
        
        $valores = $this->service->buscaValores($row)[0];
        
        $itemCole = $this->service->getItemColeNegociacao($row)[0];

        $percents = [];
        
        if($row['VRPRECUNIITMAT'] == 0){
            $percents['VRPERPRECUNIITMAT'] = 0;
        }else{
            $percents['VRPERPRECUNIITMAT'] = (($itemCole['VRPRECUNIITM'] - $row['VRPRECUNIITMAT'])    / $row['VRPRECUNIITMAT'])    * 100;
        }
        
        if($itemCole['VRPRECECOITM'] == 0){
            $percents['VRPERPRECECOITMAT'] = 0;
        }else{
            $percents['VRPERPRECECOITMAT'] = (($valores['VRPRECECOITM']  - $itemCole['VRPRECECOITM']) / $itemCole['VRPRECECOITM']) * 100;
        }
        
        // if($row['VRPRECUNIITMUC'] == 0){
            $percents['VRPERPRECUNIITMUC'] = 0;
        // }else{
        //     $percents['VRPERPRECUNIITMUC'] = (($itemCole['VRPRECUNIITM'] - $row['VRPRECUNIITMUC']) / $row['VRPRECUNIITMUC'])    * 100;
        // }
        
        // if($row['VRPERPRECECOITMUC'] == 0){
        $percents['VRPERPRECECOITMUC'] = 0;
        // }else{
        //     $percents['VRPERPRECECOITMUC'] = (($valores['VRPRECECOITM']  - $row['VRPERPRECECOITMUC']) / $row['VRPERPRECECOITMUC']) * 100;
        // }
        
        $response->addDataSet(new DataSet("resp", $percents));
    }
    
    public function validacoesIniciais(DTO\Request\Row $request, DTO\Response $response)
    {
        try{
            $row = $request->getRow();
            $centralCompras = $this->service->centralCompras($row);
            $response->addDataSet(new DataSet("centralCompras", $centralCompras));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        } 
    }
    
    public function saveCollectedData(DTO\Request\Row $request, DTO\Response $response)
    {
        try{
            $row = $request->getRow();
            $this->service->saveCollectedData($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        } 
    }
        
    public function buscaCdfilial(DTO\Request\Row $request, DTO\Response $response)
    {
        try{
            $row = $request->getRow();
            $cdfilial = $this->service->buscaCollectionId($row);
            $response->addDataSet(new DataSet("cdfilial", $cdfilial));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        } 
    }
    
    public function buscaCdforneced(DTO\Request\Row $request, DTO\Response $response)
    {
        try{
            $row = $request->getRow();
            $cdforneced = $this->service->buscaCdforneced($row);
            $response->addDataSet(new DataSet("cdforneced", $cdforneced));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        } 
    }
    
    public function getReplicarDesconto(DTO\Request $request, DTO\Response $response){
        try{    
            $row = $request->getRow();
            
           $valoresReplicarDesconto = $this->service->getReplicarDesconto($row);
            
          $response->addDataSet(new DataSet("replicarDesc", $valoresReplicarDesconto));
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        } 
    }
    
    public function getIDReplicarDesconto(DTO\Request $request, DTO\Response $response){
        try{    
            $row = $request->getRow();
            
           $valoresReplicarDesconto = $this->service->getIDReplicarDesconto($row);
            
          $response->addDataSet(new DataSet("idReplicarDesc", $valoresReplicarDesconto));
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        } 
    } 
    
    public function importarXls(DTO\Request $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            $tableRows = $this->service->readXls($row);
            $rows = $this->service->importarPedidoColeta($tableRows, $row['CDFILIAL']);
            if($rows != null){
                $response->addDataSet(new DataSet("importarXLS", $rows));
            } else {
                $response->addDataSet(new DataSet("importarXLS", ["Importado com sucesso"]));
            }
            
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        } 
    }
    
}